=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2006-02-24 13:32:45 +0000
+++ bzrlib/reconcile.py	2006-02-24 23:13:20 +0000
@@ -14,25 +14,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Plugin to fix some potential data errors in a branch.
-
-This makes sure that the inventory weave's DAG of ancestry is correct so that
-attempts to fetch the branch over http, or certain merge operations cope
-correctly.
-
-This is most likely needed if you have used fetch-ghosts from bzrlib to
-resolve ghosts after a baz (or otherwise) import and you get bizarre behaviour
-when either exporting over http or when merging from other translated branches.
-"""
+"""Reconcilers are able to fix some potential data errors in a branch."""
 
 
 import bzrlib.branch
 import bzrlib.errors as errors
 import bzrlib.progress
 from bzrlib.trace import mutter
+from bzrlib.tsort import TopoSorter
 import bzrlib.ui as ui
-from bzrlib.weavefile import write_weave_v5 as w5
-
 
 
 def reconcile(dir):
@@ -47,143 +37,6 @@
     reconciler.reconcile()
 
 
-def topological_sort(graph):
-    """Topological sort a graph.
-
-    graph -- sequence of pairs of node->parents_list.
-
-    The result is a list of node names, such that all parents come before
-    their children.
-
-    Nodes at the same depth are returned in sorted order.
-
-    node identifiers can be any hashable object, and are typically strings.
-    """
-    sorter = TopoSorter()
-    sorter.sort(graph)
-    return sorter._work_queue
-
-
-class TopoSorter(object):
-
-    def sort(self, graph):
-        # the total set of revisions to process
-        self._rev_graph = dict(graph)
-        ### if debugging:
-        # self._original_graph = dict(graph)
-        
-        # the current line of history being processed.
-        # once we add in per inventory root ids this will involve a single
-        # pass within the revision knit at that point.
-        # these contain respectively the revision id in the call stack
-        self._rev_stack = []
-        # a set of the revisions on the stack, for cycle detection
-        self._rev_set = set()
-        # and the not_ghost_parents.
-        self._pending_stack = []
-        # actual queue to reinsert. This is a single pass queue we can use to 
-        # regenerate the inventory versioned file, and holds
-        # (rev_id, non_ghost_parents) entries
-        self._work_queue = []
-        # this is a set for fast lookups of queued revisions
-        self._work_set = set()
-        # total steps = 1 read per revision + one insert into the inventory
-
-        # now we do a depth first search of the revision graph until its empty.
-        # this gives us a topological order across all revisions in 
-        # self._all_revisions.
-        # This is flattened to avoid deep recursion:
-        # At each step in the call graph we need to know which parent we are on.
-        # we do this by having three variables for each stack frame:
-        # revision_id being descended into (self._rev_stack)
-        # current queue of parents to descend into 
-        #   (self._pending_stack)
-        # putting all the parents into a pending list, left to 
-        # right and processing the right most one each time. The same parent
-        # may occur at multiple places in the stack - thats fine. We check its
-        # not in the output before processing. However revisions cannot
-        # appear twice.
-        # the algorithm is roughly:
-        # for revision, parents in _rev_graph.items():
-        #   if revision in self._all_revisions:
-        #     continue
-        #   add_revision_parents(revision, parents)
-        #  def add_revision_parents(revision, parents)
-        #    fpr parent in parents:
-        #       add_revision_parents(parent, parent.parents)
-        #   self._all_revisions.append(revision)
-        #   self._all_revision_parents.appent(parents)
-        # however we tune this to visit fragmens of the graph
-        # and not double-visit entries.
-        # nevertheless the output is a 
-        # self._work_queue of revisions, nonghost parents in 
-        # topological order and
-        # a self._work_set which is a complete set of revisions.
-        while self._rev_graph:
-            rev_id, parents = self._rev_graph.iteritems().next()
-            # rev_id
-            self._stack_revision(rev_id, parents)
-            while self._rev_stack:
-                # loop until this call completes.
-                parents_to_visit = self._pending_stack[-1]
-                # if all parents are done, the revision is done
-                if not parents_to_visit:
-                    # append the revision to the topo sorted list
-                    self._unstack_revision()
-                else:
-                    while self._pending_stack[-1]:
-                        # recurse depth first into a single parent 
-                        next_rev_id = self._pending_stack[-1].pop()
-                        if next_rev_id in self._work_set:
-                            # this parent was completed by a child on the
-                            # call stack. skip it.
-                            continue
-                        # push it into the call stack
-                        self._stack_revision(next_rev_id, self._rev_graph[next_rev_id])
-                        # and do not continue processing parents until this 'call' 
-                        # has recursed.
-                        break
-            
-                # we are now recursing down a call stack.
-                # its a width first search which 
-
-###        Useful if fiddling with this code.
-###        # cross check
-###        for index in range(len(self._work_queue)):
-###            rev = self._work_queue[index]
-###            for left_index in range(index):
-###                if rev in self.original_graph[self._work_queue[left_index]]:
-###                    print "revision in parent list of earlier revision"
-###                    import pdb;pdb.set_trace()
-
-    def _stack_revision(self, rev_id, parents):
-        """Add rev_id to the pending revision stack."""
-        # detect cycles:
-        if rev_id in self._rev_set:
-            # we only supply the revisions that led to the cycle. This isn't
-            # minimal though... but it is usually a subset of the entire graph
-            # and easier to debug.
-            raise errors.GraphCycleError(self._rev_set)
-
-        self._rev_stack.append(rev_id)
-        self._rev_set.add(rev_id)
-        self._pending_stack.append(list(parents))
-
-    def _unstack_revision(self):
-        """A revision has been completed.
-
-        The revision is added to the work queue, and the data for
-        it popped from the call stack.
-        """
-        rev_id = self._rev_stack.pop()
-        self._rev_set.remove(rev_id)
-        self._pending_stack.pop()
-        self._work_queue.append(rev_id)
-        self._work_set.add(rev_id)
-        # and remove it from the rev graph as its now complete
-        self._rev_graph.pop(rev_id)
-
-
 class Reconciler(object):
     """Reconcilers are used to reconcile existing data.
 
@@ -233,13 +86,9 @@
             # put a revision into the graph.
             self._graph_revision(rev_id)
 
-        ordered_rev_ids = topological_sort(self._rev_graph.items())
-        self._work_queue = [(rev_id, self._rev_graph[rev_id]) for 
-                            rev_id in ordered_rev_ids]
-
         # we have topological order of revisions and non ghost parents ready.
-        while self._work_queue:
-            rev_id, parents = self._work_queue.pop(0)
+        for rev_id in TopoSorter(self._rev_graph.items()).iter_topo_order():
+            parents = self._rev_graph[rev_id]
             # double check this really is in topological order.
             unavailable = [p for p in parents if p not in new_inventory]
             assert len(unavailable) == 0

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2006-02-23 04:08:56 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2006-02-24 23:13:20 +0000
@@ -1,17 +1,15 @@
 # Copyright (C) 2006 by Canonical Ltd
-# Authors: Robert Collins <robert.collins@canonical.com>
-# -*- coding: utf-8 -*-
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=== modified file 'bzrlib/tests/test_tsort.py'
--- bzrlib/tests/test_tsort.py	2006-02-24 13:32:45 +0000
+++ bzrlib/tests/test_tsort.py	2006-02-24 23:13:20 +0000
@@ -14,49 +14,59 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
 """Tests for topological sort."""
 
 
-from bzrlib.reconcile import topological_sort
 from bzrlib.tests import TestCase
-from bzrlib.tsort import topo_sort
+from bzrlib.tsort import topo_sort, TopoSorter
 from bzrlib.errors import GraphCycleError
 
 
 class TopoSortTests(TestCase):
 
+    def assertSortAndIterate(self, graph, result_list):
+        """Check that sorting and iter_topo_order on graph works."""
+        self.assertEquals(result_list, topo_sort(graph))
+        self.assertEqual(result_list,
+                         list(TopoSorter(graph).iter_topo_order()))
+
+    def assertSortAndIterateRaise(self, exception_type, graph):
+        """Try both iterating and topo_sorting graph and expect an exception."""
+        self.assertRaises(exception_type, topo_sort, graph)
+        self.assertRaises(exception_type,
+                          list,
+                          TopoSorter(graph).iter_topo_order())
+
     def test_tsort_empty(self):
         """TopoSort empty list"""
-        self.assertEquals(topo_sort([]), [])
+        self.assertSortAndIterate([], [])
 
     def test_tsort_easy(self):
         """TopoSort list with one node"""
-        self.assertEquals(topo_sort({0: []}.items()),
-                          [0])
+        self.assertSortAndIterate({0: []}.items(), [0])
 
     def test_tsort_cycle(self):
         """TopoSort traps graph with cycles"""
-        self.assertRaises(GraphCycleError,
-                          topo_sort,
-                          {0: [1], 
-                           1: [0]}.iteritems())
+        self.assertSortAndIterateRaise(GraphCycleError,
+                                       {0: [1], 
+                                        1: [0]}.items())
 
     def test_tsort_cycle_2(self):
         """TopoSort traps graph with longer cycle"""
-        self.assertRaises(GraphCycleError,
-                          topo_sort,
-                          {0: [1], 
-                           1: [2], 
-                           2: [0]}.iteritems())
+        self.assertSortAndIterateRaise(GraphCycleError,
+                                       {0: [1], 
+                                        1: [2], 
+                                        2: [0]}.items())
                  
     def test_tsort_1(self):
         """TopoSort simple nontrivial graph"""
-        self.assertEquals(topo_sort({0: [3], 
-                                     1: [4],
-                                     2: [1, 4],
-                                     3: [], 
-                                     4: [0, 3]}.iteritems()),
-                          [3, 0, 4, 1, 2])
+        self.assertSortAndIterate({0: [3], 
+                                   1: [4],
+                                   2: [1, 4],
+                                   3: [], 
+                                   4: [0, 3]}.items(),
+                                  [3, 0, 4, 1, 2])
 
     def test_tsort_partial(self):
         """Topological sort with partial ordering.
@@ -64,63 +74,13 @@
         If the graph does not give an order between two nodes, they are 
         returned in lexicographical order.
         """
-        self.assertEquals(topo_sort([(0, []),
-                                    (1, [0]),
-                                    (2, [0]),
-                                    (3, [0]),
-                                    (4, [1, 2, 3]),
-                                    (5, [1, 2]),
-                                    (6, [1, 2]),
-                                    (7, [2, 3]),
-                                    (8, [0, 1, 4, 5, 6])]),
-                          [0, 1, 2, 3, 4, 5, 6, 7, 8, ])
-
-    def test_new_tsort_empty(self):
-        """TopoSort empty list"""
-        self.assertEquals(topological_sort([]), [])
-
-    def test_new_tsort_easy(self):
-        """TopoSort list with one node"""
-        self.assertEquals(topological_sort({0: []}.items()),
-                          [0])
-
-    def test_new_tsort_cycle(self):
-        """TopoSort traps graph with cycles"""
-        self.assertRaises(GraphCycleError,
-                          topological_sort,
-                          {0: [1], 
-                           1: [0]}.iteritems())
-
-    def test_new_tsort_cycle_2(self):
-        """TopoSort traps graph with longer cycle"""
-        self.assertRaises(GraphCycleError,
-                          topological_sort,
-                          {0: [1], 
-                           1: [2], 
-                           2: [0]}.iteritems())
-                 
-    def test_new_tsort_1(self):
-        """TopoSort simple nontrivial graph"""
-        self.assertEquals(topological_sort({0: [3], 
-                                     1: [4],
-                                     2: [1, 4],
-                                     3: [], 
-                                     4: [0, 3]}.iteritems()),
-                          [3, 0, 4, 1, 2])
-
-    def test_new_tsort_partial(self):
-        """Topological sort with partial ordering.
-
-        If the graph does not give an order between two nodes, they are 
-        returned in lexicographical order.
-        """
-        self.assertEquals(topological_sort([(0, []),
-                                    (1, [0]),
-                                    (2, [0]),
-                                    (3, [0]),
-                                    (4, [1, 2, 3]),
-                                    (5, [1, 2]),
-                                    (6, [1, 2]),
-                                    (7, [2, 3]),
-                                    (8, [0, 1, 4, 5, 6])]),
-                          [0, 1, 2, 3, 4, 5, 6, 7, 8, ])
+        self.assertSortAndIterate(([(0, []),
+                                   (1, [0]),
+                                   (2, [0]),
+                                   (3, [0]),
+                                   (4, [1, 2, 3]),
+                                   (5, [1, 2]),
+                                   (6, [1, 2]),
+                                   (7, [2, 3]),
+                                   (8, [0, 1, 4, 5, 6])]),
+                                  [0, 1, 2, 3, 4, 5, 6, 7, 8])

=== modified file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	2005-10-26 05:49:06 +0000
+++ bzrlib/tsort.py	2006-02-24 23:13:20 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical
+# (C) 2005, 2006 Canonical Limited.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,9 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import pdb
-
-from bzrlib.errors import GraphCycleError
+
+"""Topological sorting routines."""
+
+
+import bzrlib.errors as errors
+
 
 def topo_sort(graph):
     """Topological sort a graph.
@@ -30,31 +33,133 @@
 
     node identifiers can be any hashable object, and are typically strings.
     """
-    parents = {}  # node -> list of parents
-    children = {} # node -> list of children
-    for node, node_parents in graph:
-        assert node not in parents, \
-            ('node %r repeated in graph' % node)
-        parents[node] = set(node_parents)
-        if node not in children:
-            children[node] = set()
-        for parent in node_parents:
-            if parent in children:
-                children[parent].add(node)
-            else:
-                children[parent] = set([node])
-    result = []
-    while parents:
-        # find nodes with no parents, and take them now
-        no_parents = [n for n in parents if len(parents[n]) == 0]
-        no_parents.sort()
-        if not no_parents:
-            raise GraphCycleError(parents)
-        for n in no_parents:
-            result.append(n)
-            for child in children[n]:
-                assert n in parents[child]
-                parents[child].remove(n)
-            del children[n]
-            del parents[n]
-    return result
+    return TopoSorter(graph).sorted()
+
+
+class TopoSorter(object):
+
+    def __init__(self, graph):
+        """Topological sorting of a graph.
+    
+        :param graph: sequence of pairs of node_name->parent_names_list.
+                      i.e. [('C', ['B']), ('B', ['A']), ('A', [])]
+                      For this input the output from the sort or
+                      iter_topo_order routines will be:
+                      'A', 'B', 'C'
+        
+        node identifiers can be any hashable object, and are typically strings.
+
+        The graph is sorted lazily: until you iterate or sort the input is
+        not processed other than to create an internal representation.
+
+        iteration or sortign may raise GraphCycleError if a cycle is present 
+        in the graph.
+        """
+        # a dict of the graph.
+        self._graph = dict(graph)
+        ### if debugging:
+        # self._original_graph = dict(graph)
+        
+        # this is a stack storing the depth first search into the graph.
+        self._node_name_stack = []
+        # at each level of 'recursion' we have to check each parent. This
+        # stack stores the parents we have not yet checked for the node at the 
+        # matching depth in _node_name_stack
+        self._pending_parents_stack = []
+        # this is a set of the completed nodes for fast checking whether a
+        # parent in a node we are processing on the stack has already been
+        # emitted and thus can be skipped.
+        self._completed_node_names = set()
+
+    def sorted(self):
+        """Sort the graph and return as a list.
+        
+        After calling this the sorter is empty and you must create a new one.
+        """
+        return list(self.iter_topo_order())
+
+###        Useful if fiddling with this code.
+###        # cross check
+###        sorted_names = list(self.iter_topo_order())
+###        for index in range(len(sorted_names)):
+###            rev = sorted_names[index]
+###            for left_index in range(index):
+###                if rev in self.original_graph[sorted_names[left_index]]:
+###                    print "revision in parent list of earlier revision"
+###                    import pdb;pdb.set_trace()
+
+    def iter_topo_order(self):
+        """Yield the nodes of a graph in a topological order.
+    
+        :param graph: sequence of pairs of node_name->parent_names_list.
+                      i.e. [('C', ['B']), ('B', ['A']), ('A', [])]
+                      For this input the following would be yielded:
+                      'A', 'B', 'C'
+        
+        node identifiers can be any hashable object, and are typically strings.
+
+        This may raise GraphCycleError if a cycle is present in the graph.
+        
+        After finishing iteration the sorter is empty and you cannot continue
+        iteration.
+        """
+        while self._graph:
+            # now pick a random node in the source graph, and transfer it to the
+            # top of the depth first search stack.
+            node_name, parents = self._graph.popitem()
+            self._push_node(node_name, parents)
+            while self._node_name_stack:
+                # loop until this call completes.
+                parents_to_visit = self._pending_parents_stack[-1]
+                # if all parents are done, the revision is done
+                if not parents_to_visit:
+                    # append the revision to the topo sorted list
+                    # all the nodes parents have been added to the output, now
+                    # we can add it to the output.
+                    yield self._pop_node()
+                else:
+                    while self._pending_parents_stack[-1]:
+                        # recurse depth first into a single parent 
+                        next_node_name = self._pending_parents_stack[-1].pop()
+                        if next_node_name in self._completed_node_names:
+                            # this parent was completed by a child on the
+                            # call stack. skip it.
+                            continue
+                        # otherwise transfer it from the source graph into the
+                        # top of the current depth first search stack.
+                        try:
+                            parents = self._graph.pop(next_node_name)
+                        except KeyError:
+                            # if the next node is not in the source graph it has
+                            # already been popped from it and placed into the
+                            # current search stack (but not completed or we would
+                            # have hit the continue 4 lines up.
+                            # this indicates a cycle.
+                            raise errors.GraphCycleError(self._node_name_stack)
+                        self._push_node(next_node_name, parents)
+                        # and do not continue processing parents until this 'call' 
+                        # has recursed.
+                        break
+
+    def _push_node(self, node_name, parents):
+        """Add node_name to the pending node stack.
+        
+        Names in this stack will get emitted into the output as they are popped
+        off the stack.
+        """
+
+        self._node_name_stack.append(node_name)
+        self._pending_parents_stack.append(list(parents))
+
+    def _pop_node(self):
+        """Pop the top node off the stack 
+
+        The node is appended to the sorted output.
+        """
+        # we are returning from the flattened call frame:
+        # pop off the local variables
+        node_name = self._node_name_stack.pop()
+        self._pending_parents_stack.pop()
+
+        self._completed_node_names.add(node_name)
+        return node_name

